﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.Xna.Framework;

namespace DBParameterEditor
{
    public partial class Form1 : Form
    {
        // define your own class here, which holds all parameters that you want to control
        // with the data base.
        public class AppParamter
        {
            public float JumpForce;
            public float Gravity;
            public float Height;
            public float Weight;
            public Vector2 StartPosition;
            public Vector2 LevelOffset;
            public int NumOfLives;
            public int MaxItems;
            public int MinItems;
        }
        AppParamter classData = new AppParamter();

        string m_domainName;

        public Form1()
        {
            InitializeComponent();

            m_domainName = "YourDomainName";

            // gui
            classNameLabel.Text = classData.GetType().Name;

            Type type = classData.GetType();
            System.Reflection.FieldInfo[] fields = type.GetFields();
            int offset = 15;
            foreach (System.Reflection.FieldInfo field in fields)
            {
                ParameterControl control = ParameterControl.CreateParameterControl(field, classData);
                control.Top = offset;
                control.Left = 5;
                offset += control.Height;
                this.parameterGroupBox.Controls.Add(control);
            }

            // send request to db
            readFromDBButton_Click(this, null);
        }

        private void readFromDBButton_Click(object sender, EventArgs e)
        {
            ParameterProcessor.Instance.ReadFromDB<AppParamter>(classData, m_domainName, delegate()
            {
                this.Invoke(new Action(UpdateValue), null);
            });
        }

        public void UpdateValue()
        {
            foreach (Control control in parameterGroupBox.Controls)
            {
                ((ParameterControl)control).UpdateValue();
            }
        }

        private void writeButton_Click(object sender, EventArgs e)
        {
            ParameterProcessor.Instance.SaveToDB<AppParamter>(classData, m_domainName);
        }

        /// <summary>
        /// clear complete item inside SimpleDB
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void clearDBButton_Click(object sender, EventArgs e)
        {
            ParameterProcessor.Instance.DeleteItem<AppParamter>(classData, m_domainName);            
        }

        /// <summary>
        /// Create code that writes current values into the class using a SetValue method.
        /// The code is then copied into the windows clipboard. Just paste it inside your source code.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void copyValuesButton_Click(object sender, EventArgs e)
        {
            StringBuilder valueCode = new StringBuilder();
            valueCode.AppendLine("public void SetValues()");
            valueCode.AppendLine("{");

            Type type = classData.GetType();
            System.Reflection.FieldInfo[] fields = type.GetFields();
            foreach (System.Reflection.FieldInfo field in fields)
            {
                string valueString;
                if (field.FieldType == typeof(float))
                {
                    valueString = ((float)field.GetValue(classData)).ToString(System.Globalization.CultureInfo.InvariantCulture.NumberFormat)
                        + "f";
                }
                else if (field.FieldType == typeof(Vector2))
                {
                    Vector2 vec = (Vector2)field.GetValue(classData);

                    System.Globalization.NumberFormatInfo nfi
                            = System.Globalization.CultureInfo.InvariantCulture.NumberFormat;
                    valueString = String.Format("new Vector2({0}f, {1}f)", vec.X.ToString(nfi), vec.Y.ToString(nfi));
                }
                else
                {
                    // just use the default ToString for all other field types
                    // extend it if you need special handling like the types before
                    valueString = field.GetValue(classData).ToString();
                }
                valueCode.AppendFormat("    {0} = {1};\n", field.Name, valueString);
            }
            valueCode.AppendLine("}");
            // and write code into clipboard
            Clipboard.SetText(valueCode.ToString());
        }
    }
}
