﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DBParameterEditor
{
    public class TgexTextBox : TextBox
    {
        protected bool m_onlyNumbers;
        /// <summary>
        /// True if the box is only for number display.
        /// </summary>
        public bool OnlyNumbers
        {
            get { return m_onlyNumbers; }
            set { m_onlyNumbers = value; }
        }

        // events
        public event System.EventHandler TextEnter;
        public event System.EventHandler TextCancel;

        string m_enterText;

        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            switch (e.KeyChar)
            {
                case (char)(int)Keys.Escape:
                    {
                        // Reset the original text value
                        this.Text = m_enterText;
                        if (this.TextCancel != null) this.TextCancel(this, e);
                        e.Handled = true;
                        return;
                    }

                case (char)(int)Keys.Enter:
                    {
                        SendEnterEvent(e);
                        e.Handled = true;
                        return;
                    }
            }
            if (m_onlyNumbers)
            {
                string valid = "0123456789.,-";
                if (!valid.Contains(e.KeyChar.ToString()) 
                    && (e.KeyChar != (int)Keys.Back) 
                    && (e.KeyChar != (int)Keys.Space))
                {
                    e.Handled = true;
                    return;
                }
            }
            base.OnKeyPress(e);
        }
        protected override void OnLeave(EventArgs e)
        {
            SendEnterEvent(e);
            base.OnLeave(e);
        }
        protected override void OnEnter(EventArgs e)
        {
            m_enterText = this.Text;
            base.OnEnter(e);
        }        
        private void SendEnterEvent(EventArgs e)
        {
            if (this.Text != m_enterText)
            {
                if (this.TextEnter != null) this.TextEnter(this, e);
                m_enterText = this.Text;
            }
        }
    }
}